{
  ----------------------------------------------------------------------------
  -- Filename:  chklist.pas
  -- Version:   0.1.0.2
  -- Author:    Michael Drig (md)
  -- Purpose:   TCheckList is derived from TListBox and
  --            and provides a listbox with a checkbox
  --            for every entry.
  -- IMPORTANT: In order to run with windows versions
  --            older than Windows 95, STDCNTRLS.PAS
  --            must be patched as following:
  --
  --     function TListBoxStrings.Add(const S: string): Integer;
  --       ...
  -- -->   SendMessage(ListBox.Handle, LB_SETITEMDATA, Result, 0);
  --     end;
  --
  --     procedure TListBoxStrings.Insert(Index: Integer; const S: string);
  --       ...
  -- -->   SendMessage(ListBox.Handle, LB_SETITEMDATA, Index, 0);
  --     end;
  --
  ----------------------------------------------------------------------------
  -- (C) 1996 EYE Communications AG
  --          Lindenhofstrasse 30
  --          4002 Basel
  --          info@eye.ch / www.eye.ch
  ----------------------------------------------------------------------------
  -- History: 02.03.96md v0.1.0.0 Creation
  --          15.03.96md v0.1.0.1 Changed clOlive as transparent color
  --          28.06.96md v0.1.0.2 Cleanup
  ----------------------------------------------------------------------------
}

unit ChkList;

interface

uses
  WinProcs, WinTypes, Messages, Classes, Graphics, Controls, Forms, StdCtrls,
  Menus, SysUtils;

type
  TCheckList = class(TCustomListBox)
  private
    FGlyphChecked: TBitmap;
    FGlyphUnchecked: TBitmap;
    procedure SetGlyphChecked(BMP: TBitmap);
    procedure SetGlyphUnchecked(BMP: TBitmap);
  protected
    procedure DrawItem
      (Index: Integer; Rect: TRect; State: TOwnerDrawState); override;
    procedure MeasureItem(Index: Integer; var Height: Integer); override;
    procedure InvalidateIndex(Index: Integer); virtual;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Check(Index: Integer);
    procedure Uncheck(Index: Integer);
    function IsChecked(Index: Integer): Boolean;
  published
    property GlyphChecked: TBitmap read FGlyphChecked write SetGlyphChecked;
    property GylphUnchecked: TBitmap
      read FGlyphUnchecked write SetGlyphUnchecked;
    property Align;
    property BorderStyle;
    property Color;
    property Columns;
    property Ctl3D;
    property DragCursor;
    property DragMode;
    property Enabled;
    property ExtendedSelect;
    property Font;
    property IntegralHeight;
    property Items;
    property MultiSelect;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property Sorted;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMeasureItem;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
  end;

procedure Register;

implementation

procedure Register;
  {02.03.96md}
begin
  RegisterComponents('Additional', [TCheckList]);
end;

constructor TCheckList.Create(AOwner: TComponent);
  {02.03.96md}
begin
  inherited Create(AOwner);
  FGlyphChecked := TBitmap.Create;
  FGlyphUnchecked := TBitmap.Create;
  Style := lbOwnerDrawVariable;
end;

destructor TCheckList.Destroy;
  {28.06.96md}
begin
  FGlyphChecked.Free;
  FGlyphUnchecked.Free;
  inherited Destroy;
end;


procedure TCheckList.InvalidateIndex(Index: Integer);
  {02.03.96md}
  {-- Invalidate Rect of the an Item specified by Index}
var
  R: TRect;
begin
  Perform(LB_GETITEMRECT, Index, Longint(@R));
  InvalidateRect(Handle, @R, false);
end;


procedure TCheckList.Check(Index: Integer);
  {02.03.96md}
  {-- Check an Item an redraw}
begin
  Items.Objects[Index] := FGlyphChecked;
  InvalidateIndex(Index);
end;


procedure TCheckList.Uncheck(Index: Integer);
  {02.03.96md}
  {-- Uncheck an Item an redraw}
begin
  Items.Objects[Index] := FGlyphUnchecked;
  InvalidateIndex(Index);
end;

function TCheckList.IsChecked(Index: Integer): Boolean;
  {02.03.96md}
begin
  result := Items.Objects[Index] = FGlyphChecked;
end;

procedure TCheckList.SetGlyphChecked(BMP: TBitmap);
  {02.03.96md}
begin
  FGlyphChecked.Assign(BMP);
end;

procedure TCheckList.SetGlyphUnchecked(BMP: TBitmap);
  {02.03.96md}
begin
  FGlyphUnchecked.Assign(BMP);
end;


procedure TCheckList.DrawItem(Index: Integer; Rect: TRect; State: TOwnerDrawState);
  {15.03.96md}
  {-- Draw an Item. First center and draw the appropriate Bitmap, then
   -- leave some space and draw the String.
   -- New Items Objects property are nil. They are always assumed to be
   -- unchecked and set to FGlyphUnchecked}
var
  Bitmap: TBitmap;
  Top: integer;
  Offset: Integer;
begin
  inherited DrawItem(Index, Rect, State);
  Offset := 2;
  with Canvas do begin
    FillRect(Rect);
    if Items.Objects[Index] = nil then
      Items.Objects[Index] := FGlyphUnchecked;
    Bitmap := TBitmap(Items.Objects[Index]);
    Top := Rect.Top + ((Rect.Bottom - Rect.Top - Bitmap.Height) div 2);
    BrushCopy(Bounds(Rect.Left + Offset, Top, Bitmap.Width, Bitmap.Height),
      Bitmap, Bounds(0, 0, Bitmap.Width, Bitmap.Height), clOlive);
    Offset := Offset + 2 + Bitmap.Width;
    TextOut(Rect.Left + Offset, Rect.Top, Items[Index]);
  end;
end;


procedure TCheckList.MeasureItem(Index: Integer; var Height: Integer);
  {02.03.96md}
  {-- Check the height of the bitmaps and the font}
begin
  Height := FGlyphUnchecked.Height;
  if FGlyphChecked.Height > Height then
    Height := FGlyphChecked.Height;
  if (Abs(Font.Height) + 4) > Height then
    Height := Abs(Font.Height) + 4;
  inherited MeasureItem(Index, Height);
end;

end.

